@@ -47,7 +47,7 @@ class Cache():
         return self.get(url)
 
     def _url_fragment_options(self, url, sha1=None):
-        default_opts = {'sha1':sha1, 'item':None, 'reader':None, 'id':None}
+        default_opts = {'sha1':sha1, 'subpath':None, 'reader':None, 'idx':None}
         url_scheme = urlparse.urlparse(url)[0]
         url_fragment = urlparse.urlparse(url)[5]   
         opt = default_opts
@@ -59,43 +59,43 @@ class Cache():
                 opt[key] = fragment[key]                
         return opt
   
-    def read(self, filename, reader=None, id=None):
-        """Read object from filename using supplied custom reader and item id"""
-        quietprint('[viset.cache.get][HIT]: Reading "' + reader + '" encoded object with id "' + str(id) + '" from file "' + filename + '"', self._verbose)
+    def read(self, filename, reader=None, idx=None):
+        """Read filename using supplied custom reader and item index and return object"""
+        quietprint('[viset.cache.get][HIT]: Reading "' + reader + '" encoded object with index "' + str(idx) + '" from file "' + filename + '"', self._verbose)
         if reader == 'h5py':
             f = h5py.File(filename,'r')
-            obj = f[id]  # FIXME: this is a link to an hdf5 object and not a numpy object.  Deep copy?
+            obj = f[idx]  # FIXME: this is a link to an hdf5 object and not a numpy object.  Deep copy?
             return obj
         elif reader == 'universal':
             return pylab.imread(filename)
         elif reader == 'mnist':
-            return viset.library.mnist.imread(filename, int(id))
+            return viset.library.mnist.imread(filename, int(idx))
         else:
             self.quietprint('[viset.cache.read][ERROR]: reader type "' + str(reader) + '" unsupported', self._verbose)
             raise NotImplementedError()
         
-    def unpack(self, filename, sha1=None, item=None):
-        """Extract cached archive to archive directory file and delete archive"""
+    def unpack(self, filename, sha1=None, subpath=None):
+        """Extract archive to archive directory, delete archive file and return archive directory"""
         if isarchive(filename):
             (unpackdir, ext) = viset.util.splitextension(filename)
             if not path.exists(unpackdir):
                 os.makedirs(unpackdir)
             viset.download.extract(filename, unpackdir, sha1=sha1, verbose=self._verbose)            
             os.remove(filename)
-            if item is not None:
-                filename = path.join(unpackdir, item)
+            if subpath is not None:
+                filename = path.join(unpackdir, subpath)
             else:
                 filename = unpackdir    
         return filename
         
-    def load(self, filename, reader=None, id=None):
-        """Load a filename to an object"""
+    def load(self, filename, reader=None, idx=None):
+        """Load from filename to an object"""
         if isimg(filename):
             # standard image formats
             obj = pylab.imread(filename)
         elif reader is not None:
             # custom reader defined in querystring
-            obj = self.read(filename, reader=reader, id=id)  
+            obj = self.read(filename, reader=reader, idx=idx)  
         elif path.isdir(viset.util.splitextension(filename)[0]):
             # archive directory in cache
             obj = viset.util.splitextension(filename)[0] 
@@ -112,7 +112,7 @@ class Cache():
         return obj
         
     def download(self, url):        
-        """Download url to cached file according to URL scheme"""
+        """Download url to cached file according to URL scheme and return filename in cache"""
         self.free()  # garbage collection time?        
         filename = self.cachefile(urlparse.urldefrag(url)[0])
         url_scheme = urlparse.urlparse(url)[0]
@@ -128,12 +128,6 @@ class Cache():
             raise NotImplementedError('FIXME: support for URL scheme ' + url_scheme)            
         return filename        
 
-    def delete(self):
-        """Delete entire cache"""
-        quietprint('[viset.cache]: Deleting all cached data in "' + self._cacheroot + '"', self._verbose)
-        shutil.rmtree(self._cacheroot)
-        os.makedirs(self._cacheroot)        
-        
     def fetch(self, url, sha1=None):
         """Fetch a url and unpack in cache"""
         urlopts = self._url_fragment_options(url, sha1)
@@ -143,10 +137,10 @@ class Cache():
                 filename = self.download(url)
                 if isarchive(filename):
                     quietprint('[viset.cache][MISS]: extracting archive \'' + filename + '\'', self._verbose)
-                    if urlopts['item'] is not None:
-                        filename = self.unpack(filename, sha1=urlopts['sha1'], item=urlopts['item'])
+                    if urlopts['subpath'] is not None:
+                        filename = self.unpack(filename, sha1=urlopts['sha1'], subpath=urlopts['subpath'])
                     else:
-                        filename = self.unpack(filename, sha1=urlopts['sha1'], item=None)                        
+                        filename = self.unpack(filename, sha1=urlopts['sha1'], subpath=None)                        
             else:
                 filename = self.cachefile(url)            
                 quietprint('[viset.cache.get][HIT]: fetching "' + url + '" from "' + filename + '"', self._verbose)                                                     
@@ -162,7 +156,7 @@ class Cache():
             return None
         
         quietprint('[viset.cache.get][HIT]: loading "' + filename + '"', self._verbose)                                         
-        return self.load(filename, reader=urlopts['reader'], id=urlopts['id'])
+        return self.load(filename, reader=urlopts['reader'], idx=urlopts['idx'])
 
     def put(self, obj, async=False, key='obj', filename=None):
         """Put a numpy object into cache and return url""" 
@@ -174,7 +168,7 @@ class Cache():
         if filename is None:
             byteview = obj.view(numpy.uint8)
             filename = hashlib.sha1(byteview).hexdigest() # byte view sha1
-        url = 'file://' + self.abspath(filename + '.h5') + '#reader=h5py&id=' + key
+        url = 'file://' + self.abspath(filename + '.h5') + '#reader=h5py&idx=' + key
         if self.iscached(url):
             quietprint('[viset.cache.put][HIT]: "' + self.cachefile(url) + '"', self._verbose)                                                     
         else:
@@ -186,7 +180,7 @@ class Cache():
         return CachedObject(url) if async else url
         
     def get(self, url, sha1=None, async=False):
-        """Get an url from cache and return object""" 
+        """Get a url from cache with refetching on error and return object""" 
         if async:
             return CachedResult(url, sha1=sha1, verbose=self._verbose, cache=self)
         else:
@@ -205,12 +199,18 @@ class Cache():
                     raise
             
     def discard(self, url):
-        """Delete url from cache"""
+        """Delete single url from cache"""
         if path.isfile(self.cachefile(url)):
             os.remove(self.cachefile(url))
         if path.isdir(self.cachedir(url)):
             shutil.rmtree(self.cachedir(url))
-        
+
+    def delete(self):
+        """Delete entire cache"""
+        quietprint('[viset.cache]: Deleting all cached data in "' + self._cacheroot + '"', self._verbose)
+        shutil.rmtree(self._cacheroot)
+        os.makedirs(self._cacheroot)        
+                    
     def size(self, source=_cacheroot):
         """Recursively compute the size of a cache directory: http://snipplr.com/view/47686/"""
         total_size = os.path.getsize(source)
@@ -245,6 +245,10 @@ class Cache():
             self._cachesize = Pool(1).apply_async(self.size(), self._cacheroot)
             self._free_ctr = self._free_maxctr
         self._free_ctr -= 1
+
+    def cachepath(self, url):
+        """Return the filename of the url in the cache.  convenience function for for cachefile"""
+        return self.cachefile(url)
     
     def cachefile(self, url):
         """The file name of the url in the cache"""
@@ -253,8 +257,8 @@ class Cache():
         urlpath = urlparse.urlunsplit([p[0],p[1],p[2],None,None])
         urlopt = self._url_fragment_options(url)
         urlhash = self.hash(url)
-        if urlopt['item'] is not None:
-            return self.abspath(path.join(urlhash, urlopt['item']))
+        if urlopt['subpath'] is not None:
+            return self.abspath(path.join(urlhash, urlopt['subpath']))
         else:
             (filename, ext) = viset.util.splitextension(path.basename(urlpath))
             return self.abspath(str(urlhash) + str(ext))
@@ -317,6 +321,5 @@ class CachedImage(CachedObject):
     def __repr__(self):
         return str('<viset.image: obj=' + str(type(self._obj)) + ', cached=' + str(self._cache.iscached(self._url)) + ', URL=\'' + str(self._url) + '\'>')        
 
-    
 class CachedResult(CachedObject):
     pass
