@@ -24,6 +24,7 @@ class View(object):
     _node = None
     _db = None    
     _dbrow = None  # fast pytables writes
+    _dbrowidx = None  # fast pytables writes    
     
     def __init__(self, node=None, db=None):
         self._node = node
@@ -160,7 +161,7 @@ class StreamView(View):
         return imap(self.read, iter(self._id))
         
     def __getitem__(self, k):
-        return self.read(self._id[k])
+        return self.read(k)
 
     def __call__(self, *args, **kwargs):
         return self.split(*args, **kwargs)
@@ -205,10 +206,7 @@ class StreamView(View):
 
 class ImageStream(StreamView):
     class Table(tables.IsDescription):
-        id = tables.Int64Col(pos=0)   
-        url = tables.StringCol(512, pos=1)   
-        id_anno = tables.Int64Col(pos=2)   
-        n_anno = tables.Int32Col(pos=3)   
+        url = tables.StringCol(512, pos=0)   
 
     def _create(self,dbobj):
         tbl = dbobj.create_table(dbobj.root, 'image', ImageStream.Table, title='Image', filters=tables.Filters(complevel=6), expectedrows=1000000)
@@ -217,30 +215,21 @@ class ImageStream(StreamView):
         tbl.flush()
         return tbl
 
-    def write(self, id_img, relpath, n_anno, id_anno, url=None, reader=None, id=None):
-        qs = {}
-        if reader is not None:
-            qs['reader'] = reader
-        if id is not None:
-            qs['id'] = id
-        if relpath is not None:
-            qs['item'] = relpath
-        url_fragment = urlparse.unquote(urllib.urlencode(qs))
+    def write(self, url, reader=None, idx=None, subpath=None):
+        url_fragment = viset.util.dict2querystring({'reader':reader, 'idx':idx, 'subpath':subpath})
         if len(url_fragment) > 0:
             url = url + '#' + url_fragment
         if self._dbrow is None:
             self._dbrow = self._node.row            
-        self._dbrow['id'] = id_img
+            self._dbrowidx = 0
+        else:
+            self._dbrowidx += 1            
         self._dbrow['url'] = url
-        self._dbrow['n_anno'] = n_anno
-        self._dbrow['id_anno'] = id_anno
         self._dbrow.append()
-        return (id_img, url, n_anno, id_anno)
-
-    # for updating values use row.update()
+        return self._dbrowidx
     
-    def read(self, id_img):
-        imurl = self._node[id_img]['url']	
+    def read(self, idx):
+        imurl = self._node[idx]['url']	
         im = CachedImage(imurl, cache=self._db._cache, verbose=self._db._verbose)
         if self._db._async:
             return im  
@@ -255,9 +244,9 @@ class ImageStream(StreamView):
 
 class CategorizationAnnotationStream(StreamView):
     class Table(tables.IsDescription):
-        id_img = tables.Int64Col(pos=0)
-        id_category = tables.Int32Col(pos=1)  
-        category = tables.StringCol(64, pos=2)      
+        idx_image = tables.Int64Col(pos=0)
+        category = tables.StringCol(64, pos=1)              
+        idx_category = tables.Int32Col(pos=2)  
 
     def _create(self, dbobj):
         try:
@@ -272,46 +261,55 @@ class CategorizationAnnotationStream(StreamView):
         tbl.attrs.viewclass = 'CategorizationAnnotationStream' 
         dbobj.flush()
 
-    def read(self, id_anno):
-        id_img = self._node[id_anno]['id_img']
-        imurl = self._db._obj.root.image[id_img]['url']
+    def read(self, idx):
+        idx_image = self._node[idx]['idx_image']
+        imurl = self._db._obj.root.image[idx_image]['url']
         im = CachedImage(imurl, cache=self._db._cache, verbose=self._db._verbose)
-        anno = self._node[id_anno]
+        anno = self._node[idx]
         if self._db._async:
-            return (im,anno)  # numpy structured array tuple
+            return (im, anno)  # numpy structured array tuple
         else:
-            return (im.get(),anno)  # numpy structured array tuple
+            return (im.get(), anno)  # numpy structured array tuple
         
-    def write(self, id_category, category, id_img):
+    def write(self, category, idx_category, idx_image=None, imurl=None):
+        if imurl is not None and idx_image is None:
+            idx_image = self._db.image.write(imurl)
+        elif idx_image is None:
+            raise IOError('annotation requires either image index or image url')
+            
         if self._dbrow is None:
             self._dbrow = self._node.row
-        self._dbrow['id_img'] = id_img        
-        self._dbrow['id_category'] = id_category
+            self._dbrowidx = 0
+        else:
+            self._dbrowidx += 1            
+        self._dbrow['idx_image'] = idx_image        
+        self._dbrow['idx_category'] = idx_category
         self._dbrow['category'] = category
         self._dbrow.append()
-
+        return self._dbrowidx
+        
     def delete(self, *args, **kwargs):
         self._node.remove_rows(*args, **kwargs)
         pass
                     
     def labels(self):
         """Unique category ids in dataset"""        
-        return np.unique(self._node.cols.id_category[:])
+        return np.unique(self._node.cols.idx_category[:])
 
     def num_categories(self):
         return len(self.labels())
 
-    def y(self, id_category):
+    def y(self, idx_category):
         """Return a binary label vector y \in {-1,1} for dataset elements of a given category id"""        
-        id = self._node.cols.id_category[:]
-        return 2*(np.int32(id == id_category)) - 1
+        idx = self._node.cols.idx_category[:]
+        return 2*(np.int32(idx == idx_category)) - 1
 
     
 class DetectionAnnotationStream(StreamView):
     class Table(tables.IsDescription):
-        id_img = tables.Int64Col(pos=0)
-        id_category = tables.Int32Col(pos=1)  
-        category = tables.StringCol(64, pos=2)      
+        category = tables.StringCol(64, pos=0)      
+        idx_category = tables.Int32Col(pos=1)  
+        idx_image = tables.Int64Col(pos=2)
         bbox_xmin = tables.Float32Col(pos=3)
         bbox_xmax = tables.Float32Col(pos=4)
         bbox_ymin = tables.Float32Col(pos=5)
@@ -329,23 +327,27 @@ class DetectionAnnotationStream(StreamView):
         tbl.attrs.viewclass = 'DetectionAnnotationStream'
         dbobj.flush()
         
-    def write(self, id_category, category, id_img, xmin, xmax, ymin, ymax):
+    def write(self, category, idx_category, idx_image, xmin, xmax, ymin, ymax):
         if self._dbrow is None:
             self._dbrow = self._node.row
-        self._dbrow['id_category'] = id_category
+            self._dbrowidx = 0
+        else:
+            self._dbrowidx += 1                
         self._dbrow['category'] = category
-        self._dbrow['id_img'] = id_img        
+        self._dbrow['idx_category'] = idx_category
+        self._dbrow['idx_image'] = idx_image        
         self._dbrow['bbox_xmin'] = xmin
         self._dbrow['bbox_xmax'] = xmax
         self._dbrow['bbox_ymin'] = ymin
         self._dbrow['bbox_ymax'] = ymax                        
         self._dbrow.append()
+        return self._dbrowidx
 
-    def read(self, id_anno):
-        id_img = self._node[id_anno]['id_img']
-        impath = self._db._obj.root.image[id_img]['url']
+    def read(self, idx):
+        idx_image = self._node[idx]['idx_image']
+        impath = self._db._obj.root.image[idx_image]['url']
         im = CachedImage(impath, cache=self._db._cache, verbose=self._db._verbose)
-        anno = self._node[id_anno]
+        anno = self._node[idx]
         if self._db._async:
             return (im,anno)  # numpy structured array tuple
         else:
@@ -354,14 +356,14 @@ class DetectionAnnotationStream(StreamView):
 
 class SegmentationAnnotationStream(StreamView):
     class Table(tables.IsDescription):
-        id_img = tables.Int64Col(pos=0)
-        id_category = tables.Int32Col(pos=1)  
-        category = tables.StringCol(64, pos=2)      
+        category = tables.StringCol(64, pos=0)      
+        idx_category = tables.Int32Col(pos=1)  
+        idx_image = tables.Int64Col(pos=2)
         bbox_xmin = tables.Float32Col(pos=3)
         bbox_xmax = tables.Float32Col(pos=4)
         bbox_ymin = tables.Float32Col(pos=5)
         bbox_ymax = tables.Float32Col(pos=6)    
-        mask = tables.StringCol(512, pos=7)   
+        maskurl = tables.StringCol(512, pos=7)   
 
     def _create(self, dbobj):
         try:
@@ -373,40 +375,42 @@ class SegmentationAnnotationStream(StreamView):
             pass  
         tbl = dbobj.create_table(dbobj.root.annotation, 'segmentation', SegmentationAnnotationStream.Table, title='segmentation', filters=tables.Filters(complevel=6), expectedrows=1000000)        
         tbl.attrs.viewname = 'segmentation'
-        tbl.attrs.viewclass = 'SegmentationAnnotationStream'  # FIXME: why not self.__name__?
+        tbl.attrs.viewclass = 'SegmentationAnnotationStream'  
         dbobj.flush()
         
-    def write(self, relpath, id_img, category=None, url=None):
-        qs = {}
-        if relpath is not None:
-            qs['item'] = relpath
-        url_fragment = urlparse.unquote(urllib.urlencode(qs))
+    def write(self, category, idx_category, idx_image, maskurl, subpath=None):
+        url_fragment = viset.util.dict2querystring({'reader':None, 'idx':None, 'subpath':subpath})
         if len(url_fragment) > 0:
-            url = url + '#' + url_fragment
+            maskurl = maskurl + '#' + url_fragment
         if self._dbrow is None:
             self._dbrow = self._node.row
-        self._dbrow['mask'] = url
-        self._dbrow['id_img'] = id_img        
+            self._dbrowidx = 0
+        else:
+            self._dbrowidx += 1
+        self._dbrow['maskurl'] = maskurl
+        self._dbrow['idx_image'] = idx_image        
         self._dbrow['category'] = category  
+        self._dbrow['idx_category'] = idx_category          
         self._dbrow.append()
-    
-    def read(self, id_anno):
-        id_img = self._node[id_anno]['id_img']
-        imurl = self._db._obj.root.image[id_img]['url']	
+        return self._dbrowidx
+        
+    def read(self, idx):
+        idx_image = self._node[idx]['idx_image']
+        imurl = self._db._obj.root.image[idx_image]['url']	
         im = CachedImage(imurl, cache=self._db._cache, verbose=self._db._verbose)
-        maskurl = self._node[id_anno]['mask']
+        maskurl = self._node[idx]['maskurl']
         if maskurl == '':
 			mask = None
         else:
 			mask = CachedImage(maskurl, cache=self._db._cache, verbose=self._db._verbose)
-        anno = self._node[id_anno]
+        anno = self._node[idx]
         if self._db._async:
-            anno = {'id_category':anno['id_category'], 'category':anno['category'], 'mask':mask}            
+            anno = {'id_category':anno['idx_category'], 'category':anno['category'], 'mask':mask}            
             return (im,anno)  # numpy structured array tuple
         else:
             if mask is not None:
                 mask = mask.get()
-            anno = {'id_category':anno['id_category'], 'category':anno['category'], 'mask':mask}                        
+            anno = {'id_category':anno['idx_category'], 'category':anno['category'], 'mask':mask}                        
             return (im.get(),anno)  # numpy structured array tuple
 
 
@@ -430,48 +434,46 @@ class CategorizationOutputGroup(GroupView):
 
     
 class CategorizationOutputStream(StreamView):
-    class Table(CategorizationAnnotationStream.Table):
-        id_prediction = tables.Int32Col(pos=3)
+    class Table(tables.IsDescription):
+        category = tables.StringCol(64, pos=0)              
+        idx_category = tables.Int32Col(pos=1)  
+        score = tables.Float32Col(pos=2)           
+        elapsed = tables.Float32Col(pos=3)   
+        idx_image = tables.Int64Col(pos=4)
+        imurl = tables.StringCol(512, pos=5)   
 
     def _create(self, dbobj, viewname=None):
         if viewname is None:
-            viewname = timestamp()
-        self._node = dbobj.create_table(dbobj.root.output.categorization, viewname, CategorizationOutputStream.Table, title=viewname, filters=tables.Filters(complevel=6), expectedrows=1000000)        
+            viewname = '_' + timestamp()
+        self._node = dbobj.create_table(dbobj.root.output.categorization, viewname, CategorizationOutputStream.Table, title=viewname, filters=tables.Filters(complevel=6), expectedrows=1000000) 
         self._node.attrs.viewname = viewname
         self._node.attrs.viewclass = 'CategorizationOutputStream' 
         dbobj.flush()
         return self
 
-    def read(self, id_anno):
-        id_img = self._node[id_anno]['id_img']
-        imurl = self._db._obj.root.image[id_img]['url']
-        im = CachedImage(imurl, cache=self._db._cache, verbose=self._db._verbose)
-        anno = self._node[id_anno]
+    def write(self, category, idx_category, score, elapsed, idx_image, imurl, imsubpath=None, imreader=None, imreaderindex=None):
+        if self._dbrow is None:
+            self._dbrow = self._node.row
+            self._dbrowidx = 0
+        else:
+            self._dbrowidx += 1   
+        self._dbrow['category'] = category
+        self._dbrow['idx_category'] = idx_category
+        self._dbrow['score'] = score
+        self._dbrow['elapsed'] = elapsed        
+        self._dbrow['imurl'] = viset.util.join_fragment(imurl, viset.util.dict2querystring({'subpath':imsubpath, 'reader':imreader, 'index':imreaderindex}))
+        if idx_image is None:
+            self._dbrow['idx_image'] = self._dbrowidx
+        else:
+            self._dbrow['idx_image'] = idx_image        
+        self._dbrow.append()
+        return self._dbrowidx
+
+    def read(self, idx):
+        im = CachedImage(self._node[idx]['imurl'], cache=self._db._cache, verbose=self._db._verbose)
+        anno = self._node[idx]
         if self._db._async:
             return (im,anno)  # numpy structured array tuple
         else:
             return (im.get(),anno)  # numpy structured array tuple
-        
-    def write(self, id_category, category, id_img):
-        if self._dbrow is None:
-            self._dbrow = self._node.row
-        self._dbrow['id_img'] = id_img        
-        self._dbrow['id_category'] = id_category
-        self._dbrow['category'] = category
-        self._dbrow.append()
-
-    def delete(self, *args, **kwargs):
-        self._node.remove_rows(*args, **kwargs)
-        pass
-                    
-    def labels(self):
-        """Unique category ids in dataset"""        
-        return np.unique(self._node.cols.id_category[:])
-
-    def num_categories(self):
-        return len(self.labels())
-
-    def y(self, id_category):
-        """Return a binary label vector y \in {-1,1} for dataset elements of a given category id"""        
-        id = self._node.cols.id_category[:]
-        return 2*(np.int32(id == id_category)) - 1
+    
